#include <stdlib.h>
#include <cstdio>
#include <stddef.h>
#include <fstream>
#include <string>
#include <vector>
#include <math.h>
#include <sstream>
#include <iostream>
#include "assert.h"
#include <getopt.h>

using namespace std;

#include "dsfmt_add.h"
#include "error_handlers.h"
//#include "probability.h"

#define DPI   6.28318530717959
#define NPTS  101
#define NSPL  10000
#define SEED  0
#define SFILE "samples.dat"
#define SAVECDF

void usage() {
  printf("Generate random samples for truncated normal or log-normal distributions.\n");
  printf("usage: genSpls [-h] [-n<nspl>] [-i<seed>] [-f<fname>] \n");
  printf(" -h         : print out this help message \n");
  printf(" -n <nspl>  : no. of random samples (default=%d) \n",NSPL);
  printf(" -i <seed>  : rng seed (default=%d) \n",SEED);
  printf(" -f <fname> : file name for output samples (default=%s) \n",SFILE);
  return;
}

int main(int argc, char *argv[]) {

  int  npts  = NPTS ;
  int  nspl  = NSPL ;
  int  iseed = SEED ;
  char *sfile = (char *)SFILE ;
  dsfmt_t RandomState; /* random number structure */

  /* Read user input */
  int c;
  while ((c=getopt(argc,(char **)argv,"hn:i:f:"))!=-1){
    switch (c) {
    case 'h':
      usage();
      return (0);
      break;
    case 'n':
      nspl = strtol(optarg, (char **)NULL,0);
      break;
    case 'i':
      iseed = strtol(optarg, (char **)NULL,0);
      break;
    case 'f':
      sfile = optarg;
      break;
   }
  }


  /* Read conditional CDFs */
  vector<double> x1(npts);
  vector<double> f1(npts);
  vector<double> x2(npts*npts);
  vector<double> f2(npts*npts);

  ifstream filein;
  string str;
  filein.open("x1f1.dat");
  for (int i=0;i<npts;i++) {
    getline(filein, str); stringstream ss(str); ss>>x1[i]>>f1[i];
  }
  filein.close();
  filein.open("x2.dat");
  for (int i=0;i<npts;i++) {
    getline(filein, str); stringstream ss(str);
    for (int j=0;j<npts;j++) ss>>x2[i*npts+j];
  }
  filein.close();
  filein.open("f2.dat");
  for (int i=0;i<npts;i++) {
    getline(filein, str); stringstream ss(str);
    for (int j=0;j<npts;j++) ss>>f2[i*npts+j];
  }
  filein.close();

  /* generate and output samples */
  ofstream sout(sfile);
  if(!sout){ printf("Error : Could not open file %s\n",sfile) ; exit(1) ;}
  dsfmt_init_gen_rand(&RandomState,iseed);
  for ( int i=0; i<nspl; i++ ) {
    double uxi1 = dsfmt_genrand_urv(&RandomState);
    double uxi2 = dsfmt_genrand_urv(&RandomState);
    int    i1 = 0;
    for ( int j=1; j<npts; j++ )
      if ( uxi1 < f1[j] ) { i1 = j-1; break; }
    int i1p1=i1+1;
    double xint = x1[i1]+(x1[i1p1]-x1[i1])*(uxi1-f1[i1])/(f1[i1p1]-f1[i1]);
    int    j1 = 0, j2=0;
    for ( int j=1; j<npts; j++ )
      if ( uxi2 < f2[i1*npts+j] ) { j1 = j-1; break; }
    for ( int j=1; j<npts; j++ )
      if ( uxi2 < f2[i1p1*npts+j] ) { j2 = j-1; break; }
    int ishift = i1*npts+j1;
    double y11 = x2[ishift]+(x2[ishift+1]-x2[ishift])
                 *(uxi2-f2[ishift])/(f2[ishift+1]-f2[ishift]);
    ishift = i1p1*npts+j2;
    double y12 = x2[ishift]+(x2[ishift+1]-x2[ishift])
		    *(uxi2-f2[ishift])/(f2[ishift+1]-f2[ishift]);
    double y1 = (y11*(x1[i1p1]-xint)+y12*(xint-x1[i1]))/(x1[i1p1]-x1[i1]);
    sout << xint << " " << y1 << endl;
  }
  sout.close();

  return (0) ;

}

