#!/bin/bash

# Defaul parameters
setpar="OFF"
getPP="OFF"
plPP="OFF"
getCRPS="OFF"

# parse command-line arguments
while [[ $# -ge 1 ]]
do
key="$1"
#echo $@
case $key in
    -r|--runID)
    runID=$2
    shift
    ;;
    --setpar)
    setpar=ON
    ;;
    --getPP)
    getPP=ON
    ;;
    --plPP)
    plPP=ON
    ;;
    --getCRPS)
    getCRPS=ON
    ;;
    *)
    # unknown option
    echo "Unknown key: ${key}"
    exit
    ;;
esac
shift
done

# Directories
scrdir=${PWD}"/../../scripts/forUQ"
datdir=${PWD}"/../../data"
daldir=${PWD}"/../../lib/dalec"

# Echo setup
echo "-------------------------------------------------"
echo "Processing:    "${runID}""
echo " - Set input parameters:           $setpar"
echo " - Compute posterior predictive:   $getPP"
echo " - Plot posterior predictive data: $plPP"
echo " - Compute CRPS:                   $getCRPS"
echo "-------------------------------------------------"

# Set params for posterior predictive
if [ "$setpar" == "ON" ]
then
  python ${scrdir}/saveParams.py ${runID} addNEEsig
  ln -fs ${datdir}/US-Ha1_drivers.csv ${runID}/.
  cd ${runID}
  if [ "$runID" == "dst_d12" ]
  then
    ${daldir}/dalec18.exe
  else
    ${daldir}/dalec23.exe
  fi
  cd ..
fi

# Compute posterior predictive values
if [ "$getPP" == "ON" ]
then
  ln -fs ${datdir}/US-Ha1_NEE_daily.txt ${runID}/.
  python ${scrdir}/getPPdata.py ${runID} addNEEsig
fi

# Plot pp
if [ "$plPP" == "ON" ]
then
  cd ${runID}
  if [ "$runID" == "dst_d12" ]
  then
    python ${scrdir}/plPP.py 18 pp
  else
    python ${scrdir}/plPP.py 23 pp
  fi
fi

# get CRPS values
if [ "$getCRPS" == "ON" ]
then
  cd ${runID}
  python ${scrdir}/getCRPS.py
fi


