#!/usr/bin/env python

import os, sys
import numpy as npy
import h5py
import matplotlib.pyplot as plt
from   scipy.stats import norm,lognorm

npar  = 23

#------------------------------------------------
#  Setup
#
pnames=npy.array(["astem", "br\_mr", "br\_lit", "br\_som", "dr", "gdd_min","gdd_max", "laimax", "leaffall",
                  "lma", "nue", "q10\_mr", "q10\_hr", "rg\_frac", "tleaf", "troot", "tstem", "tsmin","vc1","vc2","vc3","sc1","sc2"])

nrmFctrs = npy.array([0.998365, 0.999968, 0.999998, 0.999205, 0.999571, 0.999571, 0.999571, 0.999999,  0.5, 1.0, 1.0, 1.0]); 
lnFctrs  = npy.array([0.997621, 0.99797,  0.999749, 0.998907, 0.99437,  0.998907, 0.999484, 0.999484, 1.0]);
jntFctr  = 0.974956 ;

nrmMuSig = npy.array([0.7,0.085,
            4.0,0.5,
            80.0,13.0,
            7.0,1.9,
            2.0,0.3,
            2.0,0.3,
            0.2,0.045,
            5.0,1.0,
            0.0,20.0,
            9000.0,1800.0,
            1500.0,300.0,
            8800.0,1760.0]);

lnMuSig=npy.array([-7.87152, 1.15707,
         -6.46416, 0.359009,
         -9.21643, 0.290978,
         -6.51833, 0.624036,
         -1.93689, 0.604725,
         -4.21575, 0.624036,
         -7.32244, 0.43231,
         -9.62503, 0.43231,
          2.94686, 0.802669]);

nrmList =[0, 7, 9, 10, 11, 12, 13, 17, 18, 19, 20, 22];
lnList  =[1, 2, 3, 4,  8,  14, 15, 16, 21];

# Read 1D pdfs
x1D=[]
p1D=[]
for i in range(1,npar+1):
    file = h5py.File("dens_"+str(i)+".h5", "r")   
    x1D.append(npy.array(file["x"  ]))
    p1D.append(npy.array(file["p1D"]))
    file.close()

# Read 2D pdf for gdd_min/gdd_max
file = h5py.File("dens_6_7.h5", "r")   
x2D=npy.array(file["x"  ])
y2D=npy.array(file["y"  ])
p2D=npy.array(file["p2D"])
file.close()

# Get Dkl for gdd
p2Dpr=npy.zeros(x2D.shape)
for i in range(x2D.shape[0]):
    for j in range(x2D.shape[1]):
        p2Dpr[i,j] =  npy.exp(-(x2D[i,j]-100.0)**2/(2.0*24.0*24.0))/npy.sqrt(2.0*npy.pi*24.0*24.0)
        p2Dpr[i,j] *= npy.exp(-(y2D[i,j]-200.0)**2/(2.0*45.0*45.0))/npy.sqrt(2.0*npy.pi*45.0*45.0)
        p2Dpr[i,j] /= jntFctr;


p2D[npy.where(p2D<1.e-12)]=1.e-12
dsum = 0.0;
for i in range(x2D.shape[0]-1):
    for j in range(x2D.shape[1]-1):
        c1=p2D[i,  j  ]*npy.log(p2D[i,  j  ]/p2Dpr[i,  j  ])
        c2=p2D[i+1,j  ]*npy.log(p2D[i+1,j  ]/p2Dpr[i+1,j  ])
        c3=p2D[i,  j+1]*npy.log(p2D[i,  j+1]/p2Dpr[i,  j+1])
        c4=p2D[i+1,j+1]*npy.log(p2D[i+1,j+1]/p2Dpr[i+1,j+1])        
        dsum += 0.25*(c1+c2+c3+c4)*(x2D[i,j+1]-x2D[i,j])*(y2D[i+1,j]-y2D[i,j])

Dkl_gdd=dsum

# Get Dkl for other params
Dkl = npy.zeros(max(nrmList+lnList)+1)
jnm = -1
jln = -1
for i in nrmList+lnList:
    p1D[i][npy.where(p1D[i]<1.e-12)]=1.e-12
    dsum = 0.0
    if npy.where(npy.array(nrmList)==i)[0].shape[0]>0:
        # normal prior
        #print "normal prior:",i
        jnm += 1;
        mu  = nrmMuSig[2*jnm];
        sig = nrmMuSig[2*jnm+1];
        fct = nrmFctrs[jnm];
        for k in range(x1D[i].shape[0]-1):
            p1Dpr1 = norm.pdf(x1D[i][k  ],loc=mu,scale=sig)/fct;
            p1Dpr2 = norm.pdf(x1D[i][k+1],loc=mu,scale=sig)/fct;
            p1Dpr1 = max(p1Dpr1,1.e-12)
            p1Dpr2 = max(p1Dpr2,1.e-12)
            c1=p1D[i][k,  ]*npy.log(p1D[i][k  ]/p1Dpr1);
            c2=p1D[i][k+1 ]*npy.log(p1D[i][k+1]/p1Dpr2);
            if (x1D[i][k]>=0):
                dsum += 0.5*(c1+c2 )*(x1D[i][k+1]-x1D[i][k])
        Dkl[i] = dsum
    else:
        # log-normal prior
        #print "log-normal prior:",i
        jln += 1;
        mu  = lnMuSig[2*jln  ];
        sig = lnMuSig[2*jln+1];
        fct = lnFctrs[  jln  ];
        for k in range(x1D[i].shape[0]-1):
            p1Dpr1 = lognorm.pdf(x1D[i][k  ],sig,scale=npy.exp(mu))/fct;
            p1Dpr2 = lognorm.pdf(x1D[i][k+1],sig,scale=npy.exp(mu))/fct;
            p1Dpr1 = max(p1Dpr1,1.e-12)
            p1Dpr2 = max(p1Dpr2,1.e-12)
            c1=p1D[i][k,  ]*npy.log(p1D[i][k  ]/p1Dpr1);
            c2=p1D[i][k+1 ]*npy.log(p1D[i][k+1]/p1Dpr2);
            dsum += 0.5*(c1+c2)*(x1D[i][k+1]-x1D[i][k])
        Dkl[i] = dsum


pnames[5]='gdd'
pnames=npy.delete(pnames,6)
Dkl[5] = Dkl_gdd
Dkl=npy.delete(Dkl,6)
npy.savez('Dkl_23.npz',pnames=pnames,Dkl=Dkl)

