#!/usr/bin/env python

import sys, os
import numpy as npy
import h5py
import matplotlib.pyplot as plt
from scipy import stats, mgrid, c_, reshape, random, rot90

runID=sys.argv[1]

addNEEsig=False
if (sys.argv[2] == "addNEEsig"):
    addNEEsig=True

chainfile="chain.thin.h5"; # chain file
nthin = 1000;              # pick every 'nthin' state
nspl  = 10;                # no of GP samples for each model run
ndiag = 12;
#-load chain
file = h5py.File("../cal/"+runID+"/"+chainfile, 'r')   
chn = npy.array(file["chn"])
file.close()

#-thin chain and add leafcn=25
chn = chn[::nthin,1:-2]
chn=npy.insert(chn,8,25.0,axis=1)
nruns = chn.shape[0]

#-load model output
model=[]
for i in range(1,nruns+1):
    model.append(npy.genfromtxt(runID+"/modelNEE"+str(i)+".dat")[:,1])

print "done loading model runs"

# load observations 
ierr=os.system("awk 'NR>1{print $0}' "+runID+"/US-Ha1_NEE_daily.txt > daily.txt")
obs=npy.genfromtxt("daily.txt")
dayst=366
dayen=3365
ndays=dayen-dayst+1
print "done loading NEE observations"

# Create covariance matrix and GP samples
for i in range(chn.shape[0]):
    print 'Sample ',i,' out of',chn.shape[0]
    sigm=npy.diag([1.0 for j in range(ndays)])
    lcorr = chn[i][-1]*1.0
    for j in range(1,ndiag+1):
        corr = npy.exp(-j*j/(lcorr*lcorr));
        for k in range(ndays-j):
            sigm[k,k+j] = corr;
            sigm[k+j,k] = corr;
    sigm = sigm*npy.exp(chn[i][-2]);
    if (addNEEsig):
        sigm=sigm+npy.diag(obs[dayst:dayst+ndays,4])
    mu = [chn[i][-3] for j in range(ndays)]
    gpspl = npy.random.multivariate_normal(mu,sigm,nspl);
    if i == 0:
        ppmod = gpspl+model[i][dayst:dayst+ndays]
    else:
        ppmod = npy.concatenate((ppmod,gpspl+model[i][dayst:dayst+ndays]))

if (addNEEsig):
    npy.savez(runID+'/ppmod_pderr.npz',ppmod=ppmod)
else:
    npy.savez(runID+'/ppmod.npz',ppmod=ppmod)

