#!/usr/bin/env python

import numpy as npy
import matplotlib.pyplot as plt
import optparse

parser = optparse.OptionParser()
# main parameters
parser.add_option("-d","--ndim", dest="ndim", default=18,type="int",
                  help="No. of parameters")
parser.add_option("-v","--var", dest="var", default=None, type="string",
                  help="Name of QoI")
parser.add_option("-t","--thr", dest="thr", default=0.01, type="float",
                  help="Threshold for min(Sij)")
parser.add_option("-m","--mth", dest="mth", default=0,type="int",
                  help="Month for QoI")
parser.add_option("-f","--fnm", dest="figname", default="fig.eps", type="string",
                  help="Name of output figure")
parser.add_option("--vb", dest="verb", default=0, type="int",
                  help="Verbosity: 0-no screen output")
parser.add_option("--fid", dest="fid", default=None, type="string",
                  help="Plot label (a,b,...)")
# secondary parameters
parser.add_option("--vo", dest="vo", default=0.15, type="float",
                  help="Absolute offset (data units) of variable names")
parser.add_option("--lp", dest="lp", default=0.2, type="float",
                  help="Relative position of Sij values")
parser.add_option("--lsv", dest="lsv", default=18, type="int",
                  help="Variable label size")
options, remainder = parser.parse_args()

if options.var is None:
    print "Need to set name of QoI -> Abort!"
    quit()

ndim    = options.ndim
var     = options.var
thr     = options.thr
mth     = options.mth
verb    = options.verb
figname = options.figname
lp      = options.lp
lsv     = options.lsv
vo      = options.vo


# File name
fname='./Sij_'+var+'.npz'
 
# Parameter names
pnames=npy.array(["astem", "br\_mr", "br\_lit", "br\_som", "dr", "gdd", "laimax", "leaffall",
        "lma", "nue", "q10\_mr", "q10\_hr", "rg\_frac", "tleaf",
        "troot", "tstem", "tsmin",
        "vc1","vc2","vc3","sc1","sc2"])


din=npy.load(fname)
sij=din["Sij"][mth-1];

idx = npy.zeros((ndim*(ndim-1)/2,2),'int');
ij  = 0;
for i in range(ndim-1):
  for j in range(i+1,ndim):
      if (sij[i,j] > thr):
        idx[ij,0] = i ;
        idx[ij,1] = j ;
        print i,j,sij[i,j]
        ij = ij+1

npar = ij
print npar
#quit()

pairs=idx[:npar,:]
sijt=sij[idx[:npar,0],idx[:npar,1]]
sidx=npy.array(npy.unique(pairs.flatten()),dtype='int')
npar  = len(sidx)
npair = len(pairs)

if verb>0:
    for i in sidx: print "%d: %s"%(i,pnames[i].upper())
    if verb>1:
        for i in range(npair):
            print "%s %s %5.2f"%(pnames[pairs[i,0]].upper(),pnames[pairs[i,1]].upper(),sijt[i])

# Make circle plot
msize  = 30;
maxsij = 0.3;
angl=npy.array([0.5*npy.pi+(2.0*npy.pi*i)/npar for i in range(npar)])
fig = plt.figure(figsize=(8.5,7))
ax=fig.add_axes([0.01, 0.01, 0.98, 0.98],aspect='equal')
for i in range(npair):
    x1 = npy.cos(angl[sidx==pairs[i,0]])
    x2 = npy.cos(angl[sidx==pairs[i,1]])
    y1 = npy.sin(angl[sidx==pairs[i,0]])
    y2 = npy.sin(angl[sidx==pairs[i,1]])
    xc = x1+lp*(x2-x1); yc = y1+lp*(y2-y1);
    lw = 0.5+4.5*min((sijt[i]-thr)/(maxsij-thr),1)
    plt.plot([x1,x2],[y1,y2],'g-',linewidth=lw)
    ax.annotate(str(int(round(sijt[i]*100))), fontsize=lsv, xy=(xc, yc),
                bbox=dict(boxstyle="round", fc="palegreen"),
                xycoords='data', ha='center', va='center')

plt.plot(npy.cos(angl),npy.sin(angl),"bo",markersize=msize,markeredgecolor='k',markeredgewidth=2)
for i in range(npar):
    x1 = (1+vo)*npy.cos(angl[i])
    y1 = (1+vo)*npy.sin(angl[i])
    if abs(angl[i]-0.5*npy.pi)<0.01:
        hav='center'; vav='bottom'
    elif abs(angl[i]-1.5*npy.pi)<0.01:
        hav='center'; vav='top'
    elif (abs(angl[i])<0.01) or abs(angl[i]-2*npy.pi)<0.01:
        hav='left'; vav='center'
    elif abs(angl[i]-npy.pi)<0.01:
        hav='right'; vav='center'
    elif (abs(angl[i])<0.5*npy.pi) or (angl[i]>2*npy.pi) :
        hav='left'; vav='bottom'
    elif (angl[i]>0.5*npy.pi) and (angl[i]<npy.pi):
        hav='right'; vav='bottom'
    elif (angl[i]>npy.pi) and (angl[i]<1.5*npy.pi):
        hav='right'; vav='top'
    else:
        hav='left'; vav='top'
    plt.text(x1,y1,"$"+pnames[sidx[i]].upper()+"$",
             ha=hav,va=vav,fontsize=lsv)

if options.fid is not None:
    plt.text(-1.7,1.4,options.fid,fontsize=lsv+4,ha='left',va='top')

ax.set_xlim([-1.7,1.7])
ax.set_ylim([-1.4,1.4])
fig.patch.set_visible(False)
ax.axis('off')
ax.set_xticks([])
ax.set_yticks([])
plt.savefig(figname)
